@@ -17,7 +17,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-static const char _ext2_c[] = "$Id: ext2.c,v 1.21 2001/03/15 20:35:09 adilger Exp $";
+static const char _ext2_c[] = "$Id: ext2.c,v 1.29 2004/09/30 14:01:40 sct Exp $";
 
 #define USE_EXT2_IS_DATA_BLOCK
 #include "config.h"
@@ -26,7 +26,9 @@ static const char _ext2_c[] = "$Id: ext2.c,v 1.21 2001/03/15 20:35:09 adilger Ex
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <sys/ioctl.h>
 #include "ext2.h"
+#include "ext2_fs.h"
 
 /* ext2 stuff ****************************************************************/
 
@@ -207,26 +209,6 @@ void ext2_zero_blocks(struct ext2_fs *fs, blk_t block, blk_t num)
 		ext2_brelse(bh, 1);
 	}
 }
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 
 off_t ext2_get_inode_offset(struct ext2_fs *fs, ino_t inode, blk_t *block)
 {
@@ -529,52 +511,50 @@ void ext2_zero_inode(struct ext2_fs *fs, ino_t ino)
 	ext2_write_inode(fs, ino, &inode);
 }
 
+unsigned int ext2_list_backups(struct ext2_fs *fs, unsigned int *three,
+			       unsigned int *five, unsigned int *seven)
+{
+	unsigned int *min = three;
+	int mult = 3;
+	unsigned int ret;
+
+	if (!(fs->sb.s_feature_ro_compat&EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER)) {
+		ret = *min;
+		*min += 1;
+		return ret;
+	}
+
+	if (*five < *min) {
+		min = five;
+		mult = 5;
+	}
+	if (*seven < *min) {
+		min = seven;
+		mult = 7;
+	}
+
+	ret = *min;
+	*min *= mult;
+
+	return ret;
+}
+
 int ext2_bg_has_super(struct ext2_fs *fs, int group)
 {
+	unsigned int three = 1, five = 5, seven = 7;
+	unsigned int grp;
+
 	if (!(fs->flags & FL_SPARSE))
 		return 1;
 
-	/* This lists all of the possible groups in a sparse-super filesystem
-	 * that have a superblock.  It is not possible to have more groups with
-	 * superblocks, as we run out of blocks to hold group descriptors in,
-	 * or we run out of room in s_blocks_count.
-	 *
-	 * Added group numbers for maximum possible groups if we have 64-bit
-	 * block numbers and fill the sparse groups with GDT tables (less
-	 * superblock, inode/block bitmaps, and 1 inode table block):
-	 *
-	 * 1k blocksize => 8188 GDT blocks * 32 groups/block = 262016 groups
-	 *              => 262016 groups * 8192 blocks/group = 2047 GB
-	 * 2k blocksize => 16380 blocks * 64 groups/block = 1048320 groups
-	 *              => 1048320 groups * 16384 blocks/group = 32760 GB
-	 * 4k blocksize => 32764 blocks * 128 groups/block = 4193792 groups
-	 *              => 4193792 groups * 32768 blocks/group = 524224 GB
-	 * 8k blocksize => 65532 blocks * 256 groups/block = 16776192 groups
-	 *              => 16776192 groups * 65536 blocks/group = 8388608 GB
-	 */
-	switch(group) {
-		case        0:	case        1:
-		case        3:	case        5:	case        7:
-		case        9:	case       25:
-		case       27:			case       49:
-		case       81:	case      125:
-		case      243:			case      343:
-				case      625:
-		case      729:
-		case     2187:			case     2401:
-		case     6561:	case    15625:	case    16807:
-		case    19683:
-		case    59049:	case    78125:	case   117649:
-		case   177147:	case   390625:
-		case   531441:			case   823543:
-		case  1594323:	case  1953125:
-		case  4782969:			case  5764801:
-				case  9765625:
-		case 14348907:
-			return 1;
-	}
+	if (group == 0)
+		return 1;
 
-	return 0;
+	while ((grp = ext2_list_backups(fs, &three, &five, &seven)) < group)
+	       /* do nothing */;
+
+
+	return (grp == group);
 }
 
 
@@ -584,9 +564,9 @@ void ext2_close(struct ext2_fs *fs)
 	ext2_journal_deinit(fs);
 #endif
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
-	ext2_commit_metadata(fs, EXT2_META_PRIMARY | EXT2_META_BACKUP);
+	ext2_commit_metadata(fs, EXT2_META_BACKUP);
 	ext2_sync(fs);
 
 	ext2_bcache_deinit(fs);
@@ -612,8 +592,14 @@ void ext2_commit_metadata(struct ext2_fs *fs, int copies)
 	if (wmeta == EXT2_META_CLEAN)
 		return;
 
+	if (fs->flags & FL_IOCTL && wmeta & EXT2_META_PRIMARY) {
+		fprintf(stderr, "%s: writing metadata on live filesystem!\n",
+			__FUNCTION__);
+		exit(5);
+	}
+
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	fs->sb.s_r_blocks_count = (fs->r_frac * (loff_t)fs->sb.s_blocks_count)/
 				  100;
@@ -677,7 +663,7 @@ static int ext2_determine_itoffset(struct ext2_fs *fs)
 	blk_t	start, itend;
 
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	fs->itoffset = fs->gd[0].bg_inode_table - fs->sb.s_first_data_block;
 	fs->stride = 0;
@@ -780,30 +766,65 @@ void ext2_sync(struct ext2_fs *fs)
 	fs->devhandle->ops->sync(fs->devhandle->cookie);
 }
 
+int ext2_ioctl(struct ext2_fs *fs, int ioc, void *arg)
+{
+	int ret = 1;
+
+	if (fs->flags & FL_DEBUG) {
+		if (ioc == EXT2_IOC_GROUP_EXTEND)
+			printf("%s: EXTEND group to %u blocks\n",
+			       __FUNCTION__, *(__u32 *)arg);
+		else if (ioc == EXT2_IOC_GROUP_ADD)
+			printf("%s: ADD group %u\n", 
+			       __FUNCTION__, *(__u32 *)arg);
+		else
+			printf("%s: unknown IOC %x\n", __FUNCTION__, ioc);
+	}
+	if (fs->devhandle->ops->ioctl(fs->devhandle->cookie, ioc, arg) < 0) {
+		char msg[4097];
+
+		snprintf(msg, 4096, "%s: %s", fs->prog,  __FUNCTION__);
+		msg[4096] = '\0';
+		perror(msg);
+
+		ret = 0;
+	}
+
+	return ret;
+}
+
 static blk_t ext2_get_reserved(struct ext2_fs *fs, struct ext2_inode *inode)
 {
 	int	resgdblocks;
 
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	/* How many blocks to reserve for each group future GDT expansion.
 	 * This assumes Bond has the right number of blocks assigned.  It
 	 * may not be reliable unless e2fsck starts fixing Bond for us.
 	 */
 	if (inode->i_blocks == 0)
-		return 0;
+		resgdblocks = 0;
+
+	/* This is the official method of storing reserved blocks. */
+	else if (fs->sb.s_feature_compat & EXT2_FEATURE_COMPAT_RESIZE_INODE) {
+		resgdblocks = fs->sb.s_reserved_gdt_blocks;
+		if (fs->flags & FL_DEBUG)
+			printf("Found %d blocks in s_reserved_gdt_blocks\n",
+			       resgdblocks);
+	}
 
 	/* This is the new/preferred method of storing reserved blocks
 	 * We want to also allow the old method in case someone
 	 * prepared a filesystem and they now want to resize with
 	 * the new code.  This can be removed in a future release.
 	 */
-	if (inode->i_version && inode->i_version < 256) {
+	else if (inode->i_generation && inode->i_generation < 256) {
 		if (fs->flags & FL_DEBUG)
 			printf("Found %d reserved blocks in i_generation\n",
-			       inode->i_version);
-		resgdblocks = inode->i_version;
+			       inode->i_generation);
+		resgdblocks = inode->i_generation;
 	} else {
 		int sb_groups = 0;
 		int i;
@@ -819,7 +840,8 @@ static blk_t ext2_get_reserved(struct ext2_fs *fs, struct ext2_inode *inode)
 			       resgdblocks);
 	}
 
-	if (resgdblocks > 255 || resgdblocks < 0) { /* Bogosity test */
+	if (resgdblocks > fs->blocksize / 4 ||
+	    resgdblocks < 0) { /* Bogosity test */
 		fprintf(stderr,
 			"%s: wrong resgdblocks found (%d), using 0.\n"
 			"\tyou may need to run ext2prepare before online "
@@ -835,6 +857,15 @@ static blk_t ext2_get_reserved(struct ext2_fs *fs, struct ext2_inode *inode)
 	return resgdblocks;
 }
 
+#define EXT2_OPEN_COMPAT_UNSUPP ~(EXT2_FEATURE_COMPAT_DIR_PREALLOC | \
+				  EXT3_FEATURE_COMPAT_HAS_JOURNAL | \
+				  EXT2_FEATURE_COMPAT_DIR_INDEX | \
+				  EXT2_FEATURE_COMPAT_EXT_ATTR | \
+				  EXT2_FEATURE_COMPAT_RESIZE_INODE)
+#define EXT2_OPEN_RO_COMPAT_UNSUPP ~(EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER | \
+				     EXT2_FEATURE_RO_COMPAT_LARGE_FILE)
+#define EXT2_OPEN_INCOMPAT_UNSUPP ~EXT2_FEATURE_INCOMPAT_FILETYPE
+
 struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, blk_t newblocks,
 			  int flags)
 {
@@ -845,7 +876,7 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, blk_t newblocks,
 	blk_t residue;
 
 	if (flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	if ((fs = (struct ext2_fs *)malloc(sizeof(struct ext2_fs))) == NULL) {
 		fprintf(stderr, "%s: error allocating fs struct\n",
@@ -879,34 +910,34 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, blk_t newblocks,
 	}
 
 	if (!(fs->flags & FL_FORCE) &&
-	    (fs->sb.s_feature_compat & ~EXT3_FEATURE_COMPAT_HAS_JOURNAL ||
-	     (fs->sb.s_feature_incompat & ~EXT2_FEATURE_INCOMPAT_FILETYPE /* &
-	      ~(fs->flags & FL_ONLINE ? EXT3_FEATURE_INCOMPAT_RECOVER : 0)*/) ||
-	     (fs->sb.s_feature_ro_compat&~EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER)))
+	    (fs->sb.s_feature_compat & EXT2_OPEN_COMPAT_UNSUPP ||
+	     (fs->sb.s_feature_incompat & EXT2_OPEN_INCOMPAT_UNSUPP &
+	      ~(fs->flags & FL_ONLINE ? EXT3_FEATURE_INCOMPAT_RECOVER : 0)) ||
+	     (fs->sb.s_feature_ro_compat & EXT2_OPEN_RO_COMPAT_UNSUPP)))
 	{
 		fprintf(stderr,
-			"%s: ext2_open: fs has unsupported feature(s) enabled:",
-			fs->prog);
-		if(fs->sb.s_feature_compat & ~EXT3_FEATURE_COMPAT_HAS_JOURNAL)
-			fprintf(stderr, " compat %x",
-				fs->sb.s_feature_compat &
-				~EXT3_FEATURE_COMPAT_HAS_JOURNAL);
-		if(fs->sb.s_feature_ro_compat &
-		   ~EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER)
-			fprintf(stderr, " ro compat %x",
-				fs->sb.s_feature_ro_compat &
-				~EXT2_FEATURE_RO_COMPAT_SPARSE_SUPER);
-		if (fs->sb.s_feature_incompat & ~EXT2_FEATURE_INCOMPAT_FILETYPE&
-		    ~(fs->flags & FL_ONLINE ? EXT3_FEATURE_INCOMPAT_RECOVER:0))
+			"ext2_open: fs has unsupported feature(s) enabled:");
+		if(fs->sb.s_feature_compat & EXT2_OPEN_COMPAT_UNSUPP)
+			fprintf(stderr, " compat %x", fs->sb.s_feature_compat &
+						      EXT2_OPEN_COMPAT_UNSUPP);
+		if(fs->sb.s_feature_ro_compat & EXT2_OPEN_RO_COMPAT_UNSUPP)
+			fprintf(stderr, " ro_compat %x",
+						fs->sb.s_feature_ro_compat &
+						EXT2_OPEN_RO_COMPAT_UNSUPP);
+		if (!(fs->flags & FL_ONLINE) &&
+		    (fs->sb.s_feature_incompat &EXT3_FEATURE_INCOMPAT_RECOVER))
+			fprintf(stderr, " recover");
+		if (fs->sb.s_feature_incompat & EXT2_OPEN_INCOMPAT_UNSUPP &
+		    ~EXT3_FEATURE_INCOMPAT_RECOVER)
 			fprintf(stderr, " incompat %x",
 				fs->sb.s_feature_incompat &
-				~EXT2_FEATURE_INCOMPAT_FILETYPE &
-				~(fs->flags & FL_ONLINE ?
-				  EXT3_FEATURE_INCOMPAT_RECOVER : 0));
+				EXT2_OPEN_INCOMPAT_UNSUPP);
+		fprintf(stderr, "\n");
 		goto error_free_fs;
 	}
+
 	if (fs->sb.s_feature_incompat & EXT3_FEATURE_INCOMPAT_RECOVER)
-		fs->flags |= FL_EXT3;
+	      fs->flags |= FL_IOCTL;
 
 	fs->logsize = fs->sb.s_log_block_size + 10;
 	handle->ops->set_blocksize(handle->cookie, fs->logsize);
