@@ -17,7 +17,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-static const char _ext2_block_relocator_c[] = "$Id: ext2_block_relocator.c,v 1.16 2001/03/15 20:35:09 adilger Exp $";
+static const char _ext2_block_relocator_c[] = "$Id: ext2_block_relocator.c,v 1.20 2004/09/30 14:05:55 sct Exp $";
 
 #define USE_EXT2_IS_DATA_BLOCK
 #include "config.h"
@@ -243,7 +243,7 @@ static int dotindblock(struct ext2_fs *fs,
 
 
 /* Ehrm.... sorry, pedanticists! :-) */
-#define offsetof(type, field) ((int)(&(((type *)0)->field)))
+#define offsetof(type, field) ((long)(&(((type *)0)->field)))
 
 static int doinode(struct ext2_fs *fs, struct ext2_block_relocator_state *state,
 		   ino_t ino)
@@ -360,7 +360,7 @@ static int ext2_block_relocator_copy(struct ext2_fs *fs,
 	int ret = 1;
 
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	if ((buf = (unsigned char *)malloc(MAXCONT << fs->logsize)) != NULL) {
 		int num;
@@ -484,7 +484,7 @@ static int ext2_block_relocator_grab_blocks(struct ext2_fs *fs,
 	ptr = 0;
 
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	for (group = 0, start = fs->sb.s_first_data_block;
 	     group < fs->numgroups;
@@ -538,7 +538,7 @@ static int ext2_block_relocator_flush(struct ext2_fs *fs,
 		return 1;
 
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	qsort(state->block, state->usedentries, sizeof(struct ext2_block_entry),
 	      compare_block_entries);
@@ -634,7 +634,7 @@ static int ext2_block_relocate_grow(struct ext2_fs *fs,
 	int   group;
 
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	newitoffset = fs->newgdblocks + 3;
 	if (fs->newgdblocks <= fs->gdblocks + fs->resgdblocks)
@@ -653,7 +653,7 @@ static int ext2_block_relocate_grow(struct ext2_fs *fs,
 	     group < fs->numgroups;
 	     group++, start += fs->sb.s_blocks_per_group) {
 		struct ext2_buffer_head *bh;
-		blk_t olditend;
+		blk_t olditoffset, olditend;
 		blk_t newitend;
 		blk_t bb, new_bb;
 		blk_t ib, new_ib;
@@ -663,8 +663,10 @@ static int ext2_block_relocate_grow(struct ext2_fs *fs,
 		int bpg;
 
 		need_reloc = 0;
-		olditend = fs->gd[group].bg_inode_table +fs->inodeblocks -start;
+		olditoffset = fs->gd[group].bg_inode_table - start;
+		olditend = olditoffset + fs->inodeblocks;
 		newitend = newitoffset + fs->inodeblocks;
+
 		bpg = fs->sb.s_blocks_per_group;
 		if (start + bpg > fs->sb.s_blocks_count)
 			bpg = fs->sb.s_blocks_count - start;
@@ -698,8 +700,7 @@ static int ext2_block_relocate_grow(struct ext2_fs *fs,
 						       start + j);
 					continue;
 				}
-				if (j >= fs->gd[group].bg_inode_table - start &&
-				    j < olditend) {
+				if (j >= olditoffset && j < olditend) {
 					if (fs->flags & FL_DEBUG)
 						printf("skip inode table %d\n",
 						       start + j);
@@ -718,7 +719,9 @@ static int ext2_block_relocate_grow(struct ext2_fs *fs,
 							       start + j)) {
 					ext2_brelse(bh, 0);
 					return 0;
-				}
+				} else if (fs->flags & FL_DEBUG)
+					printf("relocating block %d for GDT\n",
+					       start + j);
 			}
 		}
 		if (newitend - olditend <= 0 && !need_reloc) {
@@ -745,7 +748,9 @@ static int ext2_block_relocate_grow(struct ext2_fs *fs,
 						       start + j)) {
 				ext2_brelse(bh, 0);
 				return 0;
-			}
+			} else if (fs->flags & FL_DEBUG)
+				printf("relocating block %d for itable\n",
+				       start + j);
 		}
 		/* Mark blocks for relocation that fall under the new inode
 		 * or block bitmaps.
@@ -754,23 +759,25 @@ static int ext2_block_relocate_grow(struct ext2_fs *fs,
 			printf("checking bb: old %d, new %d\n", start + bb,
 			       start + new_bb);
 		if (new_bb != bb && new_bb != ib &&
-		    (new_bb<fs->gd[group].bg_inode_table || new_bb>=olditend) &&
-		    new_bb > start + fs->newgdblocks &&
-		    check_bit(bh->data, new_bb) &&
-		    !ext2_block_relocator_mark(fs, state, start + new_bb)) {
-			ext2_brelse(bh, 0);
-			return 0;
+		    (new_bb < olditoffset || new_bb >= olditend) &&
+		    check_bit(bh->data, new_bb)) {
+			if (!ext2_block_relocator_mark(fs,state,start+new_bb)) {
+				ext2_brelse(bh, 0);
+				return 0;
+			} else if (fs->flags & FL_DEBUG)
+				printf("relocating block %d for bb\n", start+j);
 		}
 		if (fs->flags & FL_DEBUG)
 			printf("checking ib: old %d, new %d\n", start + ib,
 			       start + new_ib);
 		if (new_ib != ib && new_ib != bb &&
-		    (new_ib<fs->gd[group].bg_inode_table || new_ib>=olditend) &&
-		    new_bb > start + fs->newgdblocks &&
-		    check_bit(bh->data, new_ib) &&
-		    !ext2_block_relocator_mark(fs, state, start + new_ib)) {
-			ext2_brelse(bh, 0);
-			return 0;
+		    (new_ib < olditoffset || new_ib >= olditend) &&
+		    check_bit(bh->data, new_ib)) {
+			if (!ext2_block_relocator_mark(fs,state,start+new_ib)) {
+				ext2_brelse(bh, 0);
+				return 0;
+			} else if (fs->flags & FL_DEBUG)
+				printf("relocating block %d for ib\n", start+j);
 		}
 
 		ext2_brelse(bh, 0);
@@ -790,7 +797,7 @@ static int ext2_block_relocate_shrink(struct ext2_fs *fs,
 	blk_t start;
 
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	state->newallocoffset = fs->itoffset + fs->inodeblocks;
 	ext2_read_inode(fs, EXT2_RESIZE_INO, inode);
@@ -854,8 +861,9 @@ static int ext2_block_relocate_shrink(struct ext2_fs *fs,
 			if (j >= it && j < itend) {
 				if (delgrp)
 					continue;
-				fprintf(stderr, __FUNCTION__
-					"trying to move block %d in itable!\n",
+				fprintf(stderr, 
+					"%s: trying to move block %d in itable!\n",
+					__FUNCTION__,
 					j + start);
 				return 0;
 			}
