@@ -17,7 +17,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-static const char _ext2_buffer_c[] = "$Id: ext2_buffer.c,v 1.12 2000/10/16 17:10:35 adilger Exp $";
+static const char _ext2_buffer_c[] = "$Id: ext2_buffer.c,v 1.13 2004/09/30 14:01:41 sct Exp $";
 
 #include "config.h"
 
@@ -214,7 +214,7 @@ void ext2_bcache_dump(struct ext2_fs *fs)
 	int i;
 
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	for (i = 0; i < (1 << ext2_hash_bits); i++)
 		if (fs->bc->hash[i] != NULL) {
@@ -265,7 +265,7 @@ int ext2_bcache_init(struct ext2_fs *fs)
 	int size;
 
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	size = ext2_buffer_cache_pool_size >> (fs->logsize - 10);
 
