@@ -17,7 +17,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-static const char _ext2_inode_relocator_c[] = "$Id: ext2_inode_relocator.c,v 1.14 2000/10/04 08:59:00 adilger Exp $";
+static const char _ext2_inode_relocator_c[] = "$Id: ext2_inode_relocator.c,v 1.16 2004/09/30 14:01:41 sct Exp $";
 
 #include "config.h"
 
@@ -422,7 +422,8 @@ static int ext2_inode_relocator_grab_inodes(struct ext2_fs *fs,
 			bh = ext2_bread(fs, fs->gd[i].bg_inode_bitmap);
 			offset = i * fs->sb.s_inodes_per_group + 1;
 
-			for (j = i ? 0 : 12; j < fs->sb.s_inodes_per_group; j++)
+			for (j = i ? 0 : EXT2_GOOD_OLD_FIRST_INO;
+			     j < fs->sb.s_inodes_per_group; j++) {
 				if (!check_bit(bh->data, j)) {
 					state->inode[ptr++].dest = offset + j;
 
@@ -431,6 +432,7 @@ static int ext2_inode_relocator_grab_inodes(struct ext2_fs *fs,
 						return 1;
 					}
 				}
+			}
 
 			ext2_brelse(bh, 0);
 		}
@@ -509,7 +511,7 @@ int ext2_inode_relocate(struct ext2_fs *fs)
 	struct ext2_inode_relocator_state state;
 
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	state.usedentries = 0;
 	state.resolvedentries = 0;
