@@ -17,7 +17,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-static const char _ext2_meta_c[] = "$Id: ext2_meta.c,v 1.12 2000/10/04 08:59:00 adilger Exp $";
+static const char _ext2_meta_c[] = "$Id: ext2_meta.c,v 1.13 2004/09/30 14:01:41 sct Exp $";
 
 #include "config.h"
 
@@ -86,7 +86,7 @@ int ext2_metadata_push(struct ext2_fs *fs)
 	blk_t start;
 
 	if (fs->flags & FL_DEBUG)
-		printf(__FUNCTION__ "\n");
+		printf("%s\n", __FUNCTION__);
 
 	new_itoffset = fs->newgdblocks + 3;
 
