@@ -17,7 +17,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-static const char _ext2resize_c[] = "$Id: ext2resize.c,v 1.14 2000/10/16 17:10:35 adilger Exp $";
+static const char _ext2resize_c[] = "$Id: ext2resize.c,v 1.15 2001/04/18 22:31:21 adilger Exp $";
 
 #include "config.h"
 
@@ -30,9 +30,9 @@ static const char _ext2resize_c[] = "$Id: ext2resize.c,v 1.14 2000/10/16 17:10:3
 void usage(FILE *outfile, char *progname)
 {
 	fprintf(outfile,
-		"usage: %s [-d] [-f] [-i] [-q] [-u] [-v] [-V] device new_size[bkmgt]\n\t"
+		"usage: %s [-dfquvV] device [new_size[bkmgt]]\n\t"
 		"\t-d, --debug    : turn debug info on\n"
-		"\t-f, --force    : skip checks\n"
+		"\t-f, --force    : skip safety checks\n"
 		"\t-q, --quiet    : be quiet (print only errors)\n"
 		"\t-u, --unsafe   : skip buffer cache flushing\n"
 		"\t-v, --verbose  : be verbose\n"
@@ -66,6 +66,7 @@ void parse_args(int *argc, char **argv[], int *flags)
 		case 'u': *flags &= ~FL_SAFE; break;
 		case 'v': *flags |= FL_VERBOSE; *flags &= ~FL_QUIET; break;
 		case 'V': *flags |= FL_VERSION; *flags &= ~FL_QUIET; return;
+		default: usage(stderr, *argv[0]);
 		}
 	}
 
@@ -96,11 +97,9 @@ int main(int argc, char *argv[])
 
 	dev = argv[0];
 
-	handle = ext2_make_dev_handle_from_file(dev, progname);
-	if (handle == NULL) {
-		fprintf(stderr, "%s: can't open %s\n", progname, dev);
+	handle = ext2_make_dev_handle_from_file(dev, NULL, progname);
+	if (handle == NULL)
 		return 1;
-	}
 
 	if (argc == 2) {
 		sscanf(argv[1], "%i%c", &resize, &mod);
