@@ -1,13 +1,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-typedef unsigned int __u32;
-
-int ext2_sparse_groups(__u32 *three, __u32 *five, __u32 *seven)
+int ext2_sparse_groups(uint32_t *three, uint32_t *five, uint32_t *seven)
 {
-	__u32 *min = three;
-	__u32 mult = 3;
-	__u32 ret;
+	uint32_t *min = three;
+	uint32_t mult = 3;
+	uint32_t ret;
 
 	if (*five < *min) {
 		min = five;
@@ -26,8 +24,8 @@ int ext2_sparse_groups(__u32 *three, __u32 *five, __u32 *seven)
 
 int main(int argc, char *argv[])
 {
-	__u32 three = 1, five = 5, seven = 7;
-	__u32 max, ret;
+	uint32_t three = 1, five = 5, seven = 7;
+	uint32_t max, ret;
 
 	if (argc < 2) {
 		fprintf(stderr, "usage: %s max_groups\n", argv[0]);
